#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   Gimp-Python - allows the writing of Gimp plugins in Python.
#   Copyright (C) 1997  James Henstridge <james@daa.com.au>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Modified 2013-december-27 by j.f.garcia www.arakne.es https://www.arakne.es/en/gimp-python-console-extended-updated/
# Modified 2021/09/19 by MareroQ: change of window size ratio, by default inspect 'pdb' instead 'gimp', combining two parts [console2.py, pyconsole2.py] of the plug into one [Pyconsole2.py], add import gimpui (for handling themes)


from gimpfu import *
import gtk
import gtk.gdk as gdk
import gobject
import pango
import gtk.keysyms as _keys
import code
import sys
import keyword
import re
import gimpui 			# add MrQ
gimpui.gimp_ui_init ()  # add MrQ
# commonprefix() from posixpath
def _commonprefix(m):
    "Given a list of pathnames, returns the longest common leading component"
    if not m: return ''
    prefix = m[0]
    for item in m:
        for i in range(len(prefix)):
            if prefix[:i+1] != item[:i+1]:
                prefix = prefix[:i]
                if i == 0:
                    return ''
                break
    return prefix

class _ReadLine(object):
    class Output(object):
        def __init__(self, console, tag_name):
            object.__init__(self)
            self.buffer = console.get_buffer()
            self.tag_name = tag_name
        def write(self, text):
            pos = self.buffer.get_iter_at_mark(self.buffer.get_insert())
            self.buffer.insert_with_tags_by_name(pos, text, self.tag_name)

    class History(object):
        def __init__(self):
            object.__init__(self)
            self.items = ['']
            self.ptr = 0
            self.edited = {}

        def commit(self, text):
            if text and self.items[-1] != text:
                self.items.append(text)
            self.ptr = 0
            self.edited = {}

        def get(self, dir, text):
            if len(self.items) == 1:
                return None

            if text != self.items[self.ptr]:
                self.edited[self.ptr] = text
            elif self.edited.has_key(self.ptr):
                del self.edited[self.ptr]

            self.ptr = self.ptr + dir
            if self.ptr >= len(self.items):
                self.ptr = 0
            elif self.ptr < 0:
                self.ptr = len(self.items) - 1

            try:
                return self.edited[self.ptr]
            except KeyError:
                return self.items[self.ptr]

    def __init__(self, quit_func=None):
        object.__init__(self)

        self.quit_func = quit_func

        self.set_wrap_mode(gtk.WRAP_CHAR)
        self.modify_font(pango.FontDescription("Monospace"))

        self.buffer = self.get_buffer()
        self.buffer.connect("insert-text", self.on_buf_insert)
        self.buffer.connect("delete-range", self.on_buf_delete)
        self.buffer.connect("mark-set", self.on_buf_mark_set)
        self.do_insert = False
        self.do_delete = False

        self.stdout_tag = self.buffer.create_tag("stdout", foreground="#f046ec")
        self.stderr_tag = self.buffer.create_tag("stderr")
        self._stdout = _ReadLine.Output(self, "stdout")
        self._stderr = _ReadLine.Output(self, "stderr")

        self.cursor = self.buffer.create_mark("cursor", self.buffer.get_start_iter(), False)
        insert = self.buffer.get_insert()
        self.cursor.set_visible(True)
        insert.set_visible(False)

        self.ps = ''
        self.in_raw_input = False
        self.run_on_raw_input = None
        self.tab_pressed = 0
        self.history = _ReadLine.History()
        self.nonword_re = re.compile("[^\w\._]")

    def freeze_undo(self):
        try: self.begin_not_undoable_action()
        except: pass

    def thaw_undo(self):
        try: self.end_not_undoable_action()
        except: pass

    def raw_input(self, ps=None):
        if ps:
            self.ps = ps
        else:
            self.ps = ''

        iter = self.buffer.get_iter_at_mark(self.buffer.get_insert())

        if ps:
            self.freeze_undo()
            self.buffer.insert(iter, self.ps)
            self.thaw_undo()

        self.__move_cursor_to(iter)
        self.scroll_to_mark(self.cursor, 0.2)

        self.in_raw_input = True

        if self.run_on_raw_input:
            run_now = self.run_on_raw_input
            self.run_on_raw_input = None
            self.buffer.insert_at_cursor(run_now + '\n')

    def on_buf_mark_set(self, buffer, iter, mark):
        if mark is not buffer.get_insert():
            return
        start = self.__get_start()
        end = self.__get_end()
        if iter.compare(self.__get_start()) >= 0 and \
           iter.compare(self.__get_end()) <= 0:
                buffer.move_mark_by_name("cursor", iter)
                self.scroll_to_mark(self.cursor, 0.2)

    def __insert(self, iter, text):
        self.do_insert = True
        self.buffer.insert(iter, text)
        self.do_insert = False

    def on_buf_insert(self, buf, iter, text, len):
        if not self.in_raw_input or self.do_insert or not len:
            return
        buf.stop_emission("insert-text")
        lines = text.splitlines()
        need_eol = False
        for l in lines:
            if need_eol:
                self._commit()
                iter = self.__get_cursor()
            else:
                cursor = self.__get_cursor()
                if iter.compare(self.__get_start()) < 0:
                    iter = cursor
                elif iter.compare(self.__get_end()) > 0:
                    iter = cursor
                else:
                    self.__move_cursor_to(iter)
            need_eol = True
            self.__insert(iter, l)
        self.__move_cursor(0)

    def __delete(self, start, end):
        self.do_delete = True
        self.buffer.delete(start, end)
        self.do_delete = False

    def on_buf_delete(self, buf, start, end):
        if not self.in_raw_input or self.do_delete:
            return

        buf.stop_emission("delete-range")

        start.order(end)
        line_start = self.__get_start()
        line_end = self.__get_end()

        if start.compare(line_end) > 0:
            return
        if end.compare(line_start) < 0:
            return

        self.__move_cursor(0)

        if start.compare(line_start) < 0:
            start = line_start
        if end.compare(line_end) > 0:
            end = line_end
        self.__delete(start, end)

    def do_key_press_event(self, event, parent_type):
        if not self.in_raw_input:
            return parent_type.do_key_press_event(self, event)

        tab_pressed = self.tab_pressed
        self.tab_pressed = 0
        handled = True

        state = event.state & (gdk.SHIFT_MASK | gdk.CONTROL_MASK | gdk.MOD1_MASK)
        keyval = event.keyval

        if not state:
            if keyval == _keys.Return:
                self._commit()
            elif keyval == _keys.Up:
                self.__history(-1)
            elif keyval == _keys.Down:
                self.__history(1)
            elif keyval == _keys.Left:
                self.__move_cursor(-1)
            elif keyval == _keys.Right:
                self.__move_cursor(1)
            elif keyval == _keys.Home:
                self.__move_cursor(-10000)
            elif keyval == _keys.End:
                self.__move_cursor(10000)
            elif keyval == _keys.Tab:
                cursor = self.__get_cursor()
                if cursor.starts_line():
                    handled = False
                else:
                    cursor.backward_char()
                    if cursor.get_char().isspace():
                        handled = False
                    else:
                        self.tab_pressed = tab_pressed + 1
                        self.__complete()
            else:
                handled = False
        elif state == gdk.CONTROL_MASK:
            if keyval == _keys.u:
                start = self.__get_start()
                end = self.__get_cursor()
                self.__delete(start, end)
            elif keyval == _keys.d:
                if self.quit_func:
                    self.quit_func()
            else:
                handled = False
        else:
            handled = False
        if not handled:
            return parent_type.do_key_press_event(self, event)
        else:
            return True

    def __history(self, dir):
        text = self._get_line()
        new_text = self.history.get(dir, text)
        if not new_text is None:
            self.__replace_line(new_text)
        self.__move_cursor(0)
        self.scroll_to_mark(self.cursor, 0.2)

    def __get_cursor(self):
        return self.buffer.get_iter_at_mark(self.cursor)
    def __get_start(self):
        iter = self.__get_cursor()
        iter.set_line_offset(len(self.ps))
        return iter
    def __get_end(self):
        iter = self.__get_cursor()
        if not iter.ends_line():
            iter.forward_to_line_end()
        return iter

    def __get_text(self, start, end):
        return self.buffer.get_text(start, end, False)

    def __move_cursor_to(self, iter):
        self.buffer.place_cursor(iter)
        self.buffer.move_mark_by_name("cursor", iter)

    def __move_cursor(self, howmany):
        iter = self.__get_cursor()
        end = self.__get_cursor()
        if not end.ends_line():
            end.forward_to_line_end()
        line_len = end.get_line_offset()
        move_to = iter.get_line_offset() + howmany
        move_to = min(max(move_to, len(self.ps)), line_len)
        iter.set_line_offset(move_to)
        self.__move_cursor_to(iter)

    def __delete_at_cursor(self, howmany):
        iter = self.__get_cursor()
        end = self.__get_cursor()
        if not end.ends_line():
            end.forward_to_line_end()
        line_len = end.get_line_offset()
        erase_to = iter.get_line_offset() + howmany
        if erase_to > line_len:
            erase_to = line_len
        elif erase_to < len(self.ps):
            erase_to = len(self.ps)
        end.set_line_offset(erase_to)
        self.__delete(iter, end)

    def __get_width(self):
        if not (self.flags() & gtk.REALIZED):
            return 80
        layout = pango.Layout(self.get_pango_context())
        letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
        layout.set_text(letters)
        pix_width = layout.get_pixel_size()[0]
        return self.allocation.width * len(letters) / pix_width

    def __print_completions(self, completions):
        line_start = self.__get_text(self.__get_start(), self.__get_cursor())
        line_end = self.__get_text(self.__get_cursor(), self.__get_end())
        iter = self.buffer.get_end_iter()
        self.__move_cursor_to(iter)
        self.__insert(iter, "\n")

        width = max(self.__get_width(), 4)
        max_width = max([len(s) for s in completions])
        n_columns = max(int(width / (max_width + 1)), 1)
        col_width = int(width / n_columns)
        total = len(completions)
        col_length = total / n_columns
        if total % n_columns:
            col_length = col_length + 1
        col_length = max(col_length, 1)

        if col_length == 1:
            n_columns = total
            col_width = width / total

        for i in range(col_length):
            for j in range(n_columns):
                ind = i + j*col_length
                if ind < total:
                    if j == n_columns - 1:
                        n_spaces = 0
                    else:
                        n_spaces = col_width - len(completions[ind])
                    self.__insert(iter, completions[ind] + " " * n_spaces)
            self.__insert(iter, "\n")

        self.__insert(iter, "%s%s%s" % (self.ps, line_start, line_end))
        iter.set_line_offset(len(self.ps) + len(line_start))
        self.__move_cursor_to(iter)
        self.scroll_to_mark(self.cursor, 0.2)

    def __complete(self):
        text = self.__get_text(self.__get_start(), self.__get_cursor())
        start = ''
        word = text
        nonwords = self.nonword_re.findall(text)
        if nonwords:
            last = text.rfind(nonwords[-1]) + len(nonwords[-1])
            start = text[:last]
            word = text[last:]

        completions = self.complete(word)

        if completions:
            prefix = _commonprefix(completions)
            if prefix != word:
                start_iter = self.__get_start()
                start_iter.forward_chars(len(start))
                end_iter = start_iter.copy()
                end_iter.forward_chars(len(word))
                self.__delete(start_iter, end_iter)
                self.__insert(end_iter, prefix)
            elif self.tab_pressed > 1:
                self.freeze_undo()
                self.__print_completions(completions)
                self.thaw_undo()
                self.tab_pressed = 0

    def complete(self, text):
        return None

    def _get_line(self):
        start = self.__get_start()
        end = self.__get_end()
        return self.buffer.get_text(start, end, False)

    def __replace_line(self, new_text):
        start = self.__get_start()
        end = self.__get_end()
        self.__delete(start, end)
        self.__insert(end, new_text)

    def _commit(self):
        end = self.__get_cursor()
        if not end.ends_line():
            end.forward_to_line_end()
        text = self._get_line()
        self.__move_cursor_to(end)
        self.freeze_undo()
        self.__insert(end, "\n")
        self.in_raw_input = False
        self.history.commit(text)
        self.do_raw_input(text)
        self.thaw_undo()

    def do_raw_input(self, text):
        pass


class _Console(_ReadLine, code.InteractiveInterpreter):
    def __init__(self, locals=None, banner=None,
                 completer=None, use_rlcompleter=True,
                 start_script=None, quit_func=None):
        _ReadLine.__init__(self, quit_func)

        code.InteractiveInterpreter.__init__(self, locals)
        self.locals["__console__"] = self

        self.start_script = start_script
        self.completer = completer
        self.banner = banner

        if not self.completer and use_rlcompleter:
            try:
                import rlcompleter
                self.completer = rlcompleter.Completer()
            except ImportError:
                pass

        self.ps1 = ">>> "
        self.ps2 = "... "
        self.__start()
        self.run_on_raw_input = start_script
        self.raw_input(self.ps1)

    def __start(self):
        self.cmd_buffer = ""

        self.freeze_undo()
        self.thaw_undo()

        self.do_delete = True
        self.buffer.set_text("")
        self.do_delete = False

        if self.banner:
            iter = self.buffer.get_start_iter()
            self.buffer.insert_with_tags_by_name(iter, self.banner, "stdout")
            if not iter.starts_line():
                self.buffer.insert(iter, "\n")

    def clear(self, start_script=None):
        if start_script is None:
            start_script = self.start_script
        else:
            self.start_script = start_script

        self.__start()
        self.run_on_raw_input = start_script

    def do_raw_input(self, text):
        if self.cmd_buffer:
            cmd = self.cmd_buffer + "\n" + text
        else:
            cmd = text

        saved_stdout, saved_stderr = sys.stdout, sys.stderr
        sys.stdout, sys.stderr = self._stdout, self._stderr

        if self.runsource(cmd):
            self.cmd_buffer = cmd
            ps = self.ps2
        else:
            self.cmd_buffer = ''
            ps = self.ps1

        sys.stdout, sys.stderr = saved_stdout, saved_stderr
        self.raw_input(ps)

    def do_command(self, code):
        try:
            eval(code, self.locals)
        except SystemExit:
            if self.quit_func:
                self.quit_func()
            else:
                raise
        except:
            self.showtraceback()

    def runcode(self, code):
        if gtk.pygtk_version[1] < 8:
            self.do_command(code)
        else:
            self.emit("command", code)

    def exec_command(self, command):
        if self._get_line():
            self._commit()
        self.buffer.insert_at_cursor(command)
        self._commit()

    def complete_attr(self, start, end):
        try:
            obj = eval(start, self.locals)
            strings = dir(obj)
            if end:
                completions = {}
                for s in strings:
                    if s.startswith(end):
                        completions[s] = None
                completions = completions.keys()
            else:
                completions = strings
            completions.sort()
            return [start + "." + s for s in completions]
        except:
            return None

    def complete(self, text):
        if self.completer:
            completions = []
            i = 0
            try:
                while 1:
                    s = self.completer.complete(text, i)
                    if s:
                        completions.append(s)
                        i = i + 1
                    else:
                        completions.sort()
                        return completions
            except NameError:
                return None

        dot = text.rfind(".")
        if dot >= 0:
            return self.complete_attr(text[:dot], text[dot+1:])

        completions = {}
        strings = keyword.kwlist

        if self.locals:
            strings.extend(self.locals.keys())
        try: strings.extend(eval("globals()", self.locals).keys())
        except: pass

        try:
            exec "import __builtin__" in self.locals
            strings.extend(eval("dir(__builtin__)", self.locals))
        except:
            pass
        for s in strings:
            if s.startswith(text):
                completions[s] = None
        completions = completions.keys()
        completions.sort()
        return completions

def ReadLineType(t=gtk.TextView):
    class readline(t, _ReadLine):
        def __init__(self, *args, **kwargs):
            t.__init__(self)
            _ReadLine.__init__(self, *args, **kwargs)
        def do_key_press_event(self, event):
            return _ReadLine.do_key_press_event(self, event, t)
    gobject.type_register(readline)
    return readline

def ConsoleType(t=gtk.TextView):
    class console_type(t, _Console):
        __gsignals__ = {'command' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (object,)),'key-press-event' : 'override'}

        def __init__(self, *args, **kwargs):
            if gtk.pygtk_version[1] < 8:
                gobject.GObject.__init__(self)
            else:
                t.__init__(self)
            _Console.__init__(self, *args, **kwargs)
        def do_command(self, code):
            return _Console.do_command(self, code)
        def do_key_press_event(self, event):
            return _Console.do_key_press_event(self, event, t)
    if gtk.pygtk_version[1] < 8:
        gobject.type_register(console_type)
    return console_type

ReadLine = ReadLineType()
Console2 = ConsoleType()

def _make_window():
    window = gtk.Window()
    window.set_title("pyconsole2.py")
    swin = gtk.ScrolledWindow()
    swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
    window.add(swin)
    console = Console2(banner="Hello there!", use_rlcompleter=False, start_script="from gtk import *\n")
    swin.add(console)
    window.set_default_size(500, 400)
    window.show_all()
    if not gtk.main_level():
        window.connect("destroy", gtk.main_quit)
        gtk.main()
    return console

if __name__ == '__main__':
    if len(sys.argv) < 2 or sys.argv[1] != '-gimp':
        _make_window()

t = gettext.translation('gimp20-python', gimp.locale_directory, fallback=True)
_ = t.ugettext

PROC_NAME = 'python-fu-console2'

RESPONSE_BROWSE, RESPONSE_CLEAR, RESPONSE_SAVE = range(3)

# from gimpbaseenums.h
pdbProcType=["Internal GIMP procedure","GIMP Plug-In", "GIMP Extension", "Temporary Procedure"]
GimpPDBArgType = ['INT32', 'INT16', 'INT8', 'FLOAT', 'STRING', 'INT32ARRAY', 'INT16ARRAY',
  'INT8ARRAY', 'FLOATARRAY', 'STRINGARRAY', 'COLOR', 'ITEM', 'DISPLAY', 'IMAGE', 'LAYER',
  'CHANNEL', 'DRAWABLE', 'SELECTION', 'COLORARRAY', 'VECTORS', 'PARASITE', 'STATUS',
  'END']

def alert(msg):
  dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, msg)
  dlg.run()
  dlg.hide()

def do_console():
    import pygtk
    pygtk.require('2.0')

    import sys, gobject, gtk, gimpenums, gimpshelf, gimpui

    namespace = {'__builtins__': __builtins__, '__name__': '__main__', '__doc__': None,'pdb': gimp.pdb, 'gimp': gimp, 'shelf': gimpshelf.shelf}

    for s in gimpenums.__dict__.keys():
        if s[0] != '_':
            namespace[s] = getattr(gimpenums, s)

    class GimpConsole(Console2):
        def __init__(self, quit_func=None):
            banner = ('GIMP %s Python Console\nPython %s\n' % (gimp.pdb.gimp_version(), sys.version))
            Console2.__init__(self, locals=namespace, banner=banner, quit_func=quit_func)
        def _commit(self):
            Console2._commit(self)
            gimp.displays_flush()

    class ConsoleDialog(gimpui.Dialog):
        def __init__(self):
            gimpui.Dialog.__init__(self, title=_("Python Console"),
                role=PROC_NAME, help_id=PROC_NAME,
                buttons=(gtk.STOCK_SAVE, RESPONSE_SAVE, gtk.STOCK_CLEAR, RESPONSE_CLEAR, _("_Browse..."), RESPONSE_BROWSE, gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
            self.set_alternative_button_order((gtk.RESPONSE_CLOSE, RESPONSE_BROWSE, RESPONSE_CLEAR, RESPONSE_SAVE))
            self.cons = GimpConsole(quit_func=lambda: gtk.main_quit())
            self.connect('response', self.response)
            self.browse_dlg = None
            self.save_dlg = None

            table = gtk.Table(6, 6, True)
            table.set_homogeneous(True)
            table.set_row_spacings(2)
            table.set_col_spacings(2)
            table.show()

            lb = gtk.Label('Inspect object:')
            lb.show()
            lbFilter = gtk.Label('Show containing:')
            lbFilter.show()

            self.treestore = gtk.TreeStore(str,str)
            self.modelfilter = self.treestore.filter_new()
            self.treeview = gtk.TreeView(self.treestore)
            self.treeview.set_rules_hint(True)
            #self.treeview.connect("cursor-changed", self.timeoutFocus, self.cons)
            cols = ['Inspecting gimp','Type']
            for i in range(0,len(cols)):
                self.tvcolumn = gtk.TreeViewColumn(cols[i])
                self.treeview.append_column(self.tvcolumn)
                cell = gtk.CellRendererText()
                self.tvcolumn.pack_start(cell, True)
                self.tvcolumn.add_attribute(cell, 'text', i)
                self.tvcolumn.set_sort_column_id(0)
            self.treeview.set_reorderable(False)
            self.treeview.set_search_column(0)

            self.tree_sel = self.treeview.get_selection()
            self.tree_sel.set_mode(gtk.SELECTION_SINGLE)
            self.tree_sel.connect("changed", self.onSelectionChanged)
            self.treeview.connect("row-activated", self.actRow)

            self.entry = gtk.Entry()
            self.txFilter = gtk.Entry()
            self.txFilter.set_editable(True)
            self.txFilter.show()
            self.txFilter.connect("changed", self.filterTree)

            self.entry.set_text('pdb')

            self.entry.show()
            self.entry.connect('activate',self.inspectclick)

            btn = gtk.Button('Inspect')
            btn.show()
            btn.drag_dest_set(0, [], 0)

            btn.connect("clicked", self.inspectclick)
            scrl = gtk.ScrolledWindow()
            scrl.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
            scrl.add(self.treeview)

            self.scrltree=scrl

            scrl2 = gtk.ScrolledWindow()
            scrl2.set_shadow_type(gtk.SHADOW_IN)
            scrl2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
            self.txDoc = gtk.TextView(buffer=None)
            tb = self.txDoc.get_buffer()
            #self.h_tag = tb.create_tag("h", size_points=11, weight=pango.WEIGHT_BOLD, background='#cccccc',background_full_height=True) rem MrQ
            self.h_tag = tb.create_tag("h", size_points=11, weight=pango.WEIGHT_BOLD,background_full_height=True)
            #self.ita_tag = tb.create_tag("ita", size_points=7, style= pango.STYLE_ITALIC, background='#cccccc') rem MrQ			
            self.ita_tag = tb.create_tag("ita", size_points=7, style= pango.STYLE_ITALIC)
            self.pdbTit_tag = tb.create_tag("pt", weight=pango.WEIGHT_BOLD,left_margin=15)
            #self.pdbDoc_tag = tb.create_tag("pt2",background='#eeeeee',indent=12, justification= gtk.JUSTIFY_FILL)
            self.pdbDoc_tag = tb.create_tag("pt2",indent=12, justification= gtk.JUSTIFY_FILL)			
            self.pdbTx_tag = tb.create_tag("p",left_margin=25)
            self.pdbParam_tag = tb.create_tag("p3",left_margin=25,size_points=7, family='Monospace')
            self.pdbTx2_tag = tb.create_tag("p2",left_margin=35)
            #self.val_tag = tb.create_tag("v", foreground="#00F000", background='#000000', family='Monospace')	rem MrQ		
            self.val_tag = tb.create_tag("v", family='Monospace')
            self.txDoc.set_wrap_mode(gtk.WRAP_WORD)

            scrl2.add(self.txDoc)

            vbox = gtk.VBox(False, 12)
            vbox.set_border_width(6)
            self.vbox.pack_start(vbox)

            vbox.pack_start(table)
            self.docBuff = ''
            scrl_win = gtk.ScrolledWindow()
            scrl_win.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)

            table.attach(lb           , 4, 5,  0,  1)
            table.attach(self.entry   , 5, 6,  0,  1)
            table.attach(btn          , 4, 6,  1,  2)
            table.attach(scrl         , 4, 6,  2, 11)
            table.attach(lbFilter     , 4, 5, 11, 12)
            table.attach(self.txFilter, 5, 6, 11, 12)
            table.attach(scrl2        , 4, 6, 12, 22)
            table.attach(scrl_win     , 0, 4,  0, 22)

            self.table = table
            scrl_win.add(self.cons)
            self.set_default_size(500, 500)
            self.inspectObj('pdb')
            self.modelfilter.set_visible_func(self.visible_cb, None)
            self.treeview.set_model(self.modelfilter)
            self.label=lb

        def bufferAdd(self, tx, tag=None):
            tb = self.txDoc.get_buffer()
            p = tb.get_end_iter()
            if tag != None:
                tb.insert_with_tags( p, tx, tag)
            else:
                tb.insert(p, tx)

        def filterTree(self,extra=None):
            self.modelfilter.refilter()

        def visible_cb(self, model, iter, data):
            tx=self.txFilter.get_text()
            if tx=='':
                return True
            else:
                return str(model.get_value(iter, 0)).lower().find(tx.lower())>-1

        def onSelectionChanged(self, tree_selection):
            vals={'v':'','doc':''}
            tx = self.entry.get_text()
            self.cons.do_command(tx)
            (model, iter) = tree_selection.get_selected()
            value = model.get_value(iter,0)
            tipo = model.get_value(iter,1)
            txObj = tx+'.'+value
            if value+'' != 'clean_all':
                tmp=eval(txObj, self.cons.locals)
            else:
                tmp=None
            tb = self.txDoc.get_buffer()
            ini,end = tb.get_bounds()
            tb.delete(ini,end)
            if tipo=='str' or tipo=='tuple' or tipo=='int':
                vals['v'] = tmp
            elif tipo=='PDBFunction':
                self.bufferAdd(tmp.proc_name + " ", self.h_tag)
                self.bufferAdd(pdbProcType[int(tmp.proc_type)] + "\r", self.ita_tag)
                if tmp.proc_help!='':
                    self.bufferAdd(tmp.proc_help + "\r", self.pdbDoc_tag)
                self.bufferAdd("Parameters\r", self.pdbTit_tag)
                for n in range(tmp.nparams):
                    self.bufferAdd(GimpPDBArgType[int(tmp.params[n][0])]+' ', self.pdbParam_tag)
                    self.bufferAdd(str(tmp.params[n][1]) + "\r", self.pdbTx_tag)
                    self.bufferAdd(str(tmp.params[n][2]) + "\r", self.pdbTx2_tag)
                if len(tmp.return_vals)!=0:
                    self.bufferAdd("Return values\r", self.pdbTit_tag)
                    for n in tmp.return_vals:
                        self.bufferAdd(GimpPDBArgType[int(n[0])]+' ', self.pdbParam_tag)  
                        self.bufferAdd(str(n[1]) + "\r", self.pdbTx_tag)
                        self.bufferAdd(str(n[2]) + "\r", self.pdbTx2_tag)
                if tmp.proc_author!='':
                    self.bufferAdd("Author: ", self.pdbTit_tag)
                    self.bufferAdd(tmp.proc_author + "\r", self.pdbTx_tag)
                self.bufferAdd(tmp.proc_copyright + "\r", self.pdbTx_tag)
                self.bufferAdd(str(tmp.proc_date) + "\r", self.pdbTx_tag)
                return
            if hasattr(tmp,'__doc__'):
                if tmp.__doc__!=None:
                    vals['doc']=tmp.__doc__
            if ((vals['v']=='') and (vals['doc']=='')):
                self.bufferAdd(txObj + "\n", self.h_tag)
                self.bufferAdd(tipo + '\n')
            else:
                self.bufferAdd(txObj + "\n", self.h_tag)
                self.bufferAdd(tipo)
                if (vals['v']!=''):
                    self.bufferAdd('=')
                    self.bufferAdd(str(vals['v']),  self.val_tag)
                if (vals['doc']!=''):
                    self.bufferAdd('\n' + vals['doc']+'\n')

        def inspectObj(self, tx):
            self.treestore.clear()
            self.cons.do_command(tx)
            tmp = eval(tx, self.cons.locals)
            tbuffer = self.txDoc.get_buffer()
            self.docBuff = ''
            if hasattr(tmp,'__doc__'):
                if tmp.__doc__!=None:
                    self.docBuff=tmp.__doc__ + ''
            tbuffer.set_text(self.docBuff)
            self.treeview.get_columns()[0].set_title('Inspecting '+tx)
            all=dir(tmp)
            for ii in all:
                if hasattr(tmp,ii):
                    piter = self.treestore.append(None, [ii, ''])
            for n in self.treestore:
                v=self.treestore.get_value(n.iter,0)
                tmp=eval(tx + '.' + v, self.cons.locals)
                self.treestore.set_value(n.iter,1,type(tmp).__name__)

        def inspectclick(self, widget):
            self.inspectObj(self.entry.get_text())

        def actRow(self, treeview, path, column):
           model = treeview.get_model()
           iter = model.get_iter(path)
           buffer = self.cons.buffer
           tx=self.entry.get_text()
           fn=model.get_value(iter, 0)
           if model.get_value(iter, 1)=='PDBFunction':
                tmp=eval(tx+'.'+fn, self.cons.locals)
                params=''
                for n in range(tmp.nparams):
                    if params!='':
                        params=params+' ,'
                    params= params + str(tmp.params[n][1])
                if params!='':
                    fn=fn+'('+params+')'
                retVals=''
                if len(tmp.return_vals)!=0:
                    for n in tmp.return_vals:
                        if retVals!='':
                            retVals=retVals+', '
                        retVals=retVals+ str(n[1])
                if retVals!='':
                    tx=retVals+' = '+tx

           buffer.insert_at_cursor(tx+'.'+fn)
           # timeout para que el focus deje de estar en el treeview
           gobject.timeout_add(10, lambda: self.cons.grab_focus())
           return

        def response(self, dialog, response_id):
            if response_id == RESPONSE_BROWSE:
                self.browse()
            elif response_id == RESPONSE_CLEAR:
                self.cons.banner = None
                self.cons.clear()
            elif response_id == RESPONSE_SAVE:
                self.save_dialog()
            else:
                gtk.main_quit()
            self.cons.grab_focus()

        def browse_response(self, dlg, response_id):
            if response_id != gtk.RESPONSE_APPLY:
                dlg.hide()
                return
            proc_name = dlg.get_selected()
            if not proc_name:
                return
            proc = pdb[proc_name]
            cmd = ''
            if len(proc.return_vals) > 0:
                cmd = ', '.join([x[1].replace('-', '_') for x in proc.return_vals]) + ' = '
            cmd = cmd + 'pdb.%s' % proc.proc_name.replace('-', '_')
            if len(proc.params) > 0 and proc.params[0][1] == 'run-mode':
                params = proc.params[1:]
            else:
                params = proc.params
            cmd = cmd + '(%s)' % ', '.join([x[1].replace('-', '_') for x in params])
            buffer = self.cons.buffer
            lines = buffer.get_line_count()
            iter = buffer.get_iter_at_line_offset(lines - 1, 4)
            buffer.delete(iter, buffer.get_end_iter())
            buffer.place_cursor(buffer.get_end_iter())
            buffer.insert_at_cursor(cmd)

        def browse(self):
            if not self.browse_dlg:
                dlg = gimpui.ProcBrowserDialog(_("Python Procedure Browser"), role=PROC_NAME, buttons=(gtk.STOCK_APPLY, gtk.RESPONSE_APPLY, gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
                dlg.set_default_response(gtk.RESPONSE_APPLY)
                dlg.set_alternative_button_order((gtk.RESPONSE_CLOSE, gtk.RESPONSE_APPLY))
                dlg.connect('response', self.browse_response)
                dlg.connect('row-activated', lambda dlg: dlg.response(gtk.RESPONSE_APPLY))
                self.browse_dlg = dlg
            self.browse_dlg.present()

        def save_response(self, dlg, response_id):
            if response_id == gtk.RESPONSE_DELETE_EVENT:
                self.save_dlg = None
                return
            elif response_id == gtk.RESPONSE_OK:
                filename = dlg.get_filename()
                try:
                    logfile = open(filename, 'w')
                except IOError, e:
                    gimp.message(_("Could not open '%s' for writing: %s") % (filename, e.strerror))
                    return
                buffer = self.cons.buffer
                start = buffer.get_start_iter()
                end = buffer.get_end_iter()
                log = buffer.get_text(start, end, False)
                try:
                    logfile.write(log)
                    logfile.close()
                except IOError, e:
                    gimp.message(_("Could not write to '%s': %s") % (filename, e.strerror))
                    return
            dlg.hide()

        def save_dialog(self):
            if not self.save_dlg:
                dlg = gtk.FileChooserDialog(_("Save Python-Fu Console Output"), parent=self, action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
                dlg.set_default_response(gtk.RESPONSE_OK)
                dlg.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
                dlg.connect('response', self.save_response)
                self.save_dlg = dlg
            self.save_dlg.present()

        def run(self):
            self.show_all()
            gtk.main()

    ConsoleDialog().run()

register( PROC_NAME, N_("Interactive GIMP Python interpreter"), "Type in commands and see results", "James Henstridge", "James Henstridge", "1997-1999", N_("_Console extended"), "", [], [], do_console, menu="<Image>/Filters/Languages/Python-Fu", domain=("gimp20-python", gimp.locale_directory))

main()
# 207